from auto_deprecator.cli.auto_deprecate import deprecate_single_file

from .conftest import (
    IMPORT_STATEMENT,
    CLASS_DELARATION,
    INIT_METHOD,
    DEPRECATE_2_2_0,
    DEPRECATE_2_2_0_PROPERTY,
    INNER_CLASS,
)


def test_auto_deprecate_single_file_2_1_0(dummy_class_file):
    deprecate_single_file(filename=dummy_class_file, current="2.1.0")

    filestream = open(dummy_class_file, "r").read()
    assert filestream == (
        IMPORT_STATEMENT
        + CLASS_DELARATION
        + INIT_METHOD
        + DEPRECATE_2_2_0
        + DEPRECATE_2_2_0_PROPERTY
        + INNER_CLASS
    )


def test_auto_deprecate_single_file_2_2_0(dummy_class_file):
    deprecate_single_file(filename=dummy_class_file, current="2.2.0")

    filestream = open(dummy_class_file, "r").read()
    assert filestream == (
        IMPORT_STATEMENT + CLASS_DELARATION + INIT_METHOD + INNER_CLASS
    )


def test_auto_deprecate_single_file_2_3_0(dummy_class_file):
    deprecate_single_file(filename=dummy_class_file, current="2.3.0")

    filestream = open(dummy_class_file, "r").read()
    assert filestream == (CLASS_DELARATION + INIT_METHOD)



import pytest

from auto_deprecator import deprecate


@deprecate(expiry="2.1.0", current="2.0.0", relocate="other_func")
def simple_deprecate():
    pass


def test_no_error_simple_deprecate():
    with pytest.warns(DeprecationWarning) as warning:
        simple_deprecate()

    assert (
        'Function "simple_deprecate" will be deprecated on version '
        '2.1.0. Please use function / method "other_func"'
    ) in warning[0].message.args[0]



import importlib.util
from tempfile import NamedTemporaryFile

import pytest


IMPORT_STATEMENT = """from auto_deprecator import deprecate"""


NORMAL_FUNCTION = """


def normal_function():
    pass"""


DEPRECATE_FUNCTION_2_0_0 = """


@deprecate(expiry="2.0.0", current="2.1.0")
def deprecate_version_2_0_0():
    pass"""


DEPRECATE_FUNCTION_2_1_0 = """


@deprecate(expiry="2.1.0", current="2.1.0")
def deprecate_version_2_1_0():
    pass"""


DEPRECATE_FUNCTION_2_2_0 = """


@deprecate(expiry="2.2.0", current="2.1.0")
def deprecate_version_2_2_0():
    pass"""


@pytest.fixture(scope="session")
def function_file_str():
    return (
        IMPORT_STATEMENT
        + NORMAL_FUNCTION
        + DEPRECATE_FUNCTION_2_0_0
        + DEPRECATE_FUNCTION_2_1_0
        + DEPRECATE_FUNCTION_2_2_0
    )


@pytest.fixture
def function_file(function_file_str):
    with NamedTemporaryFile(mode="a+", suffix=".py") as tmpfile:
        tmpfile.write(function_file_str)
        tmpfile.flush()
        yield tmpfile.name


@pytest.fixture
def function_module(function_file):
    spec = importlib.util.spec_from_file_location("function", function_file)
    function_module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(function_module)
    return function_module



"""Top-level package for Auto deprecator."""
# flake8: noqa

__author__ = """Gavin Chan"""
__email__ = "gavincyi@gmail.com"
__version__ = "2020.2.0"

from .deprecate import (
    deprecate,
    check_deprecation,
)



from auto_deprecator.cli.auto_deprecate import deprecate_single_file

from .conftest import (
    IMPORT_STATEMENT,
    NORMAL_FUNCTION,
    DEPRECATE_FUNCTION_2_2_0,
)


def test_auto_deprecate_single_file_2_1_0(function_file):
    deprecate_single_file(filename=function_file, current="2.1.0")

    filestream = open(function_file, "r").read()
    assert filestream == (
        IMPORT_STATEMENT + NORMAL_FUNCTION + DEPRECATE_FUNCTION_2_2_0
    )


def test_auto_deprecate_single_file_2_2_0(function_file):
    deprecate_single_file(filename=function_file, current="2.2.0")

    filestream = open(function_file, "r").read()
    assert filestream == (NORMAL_FUNCTION).lstrip("\n")



import pytest


def test_deprecate_version_2_0_0(DummyClass):
    with pytest.raises(RuntimeError) as err:
        obj = DummyClass()
        obj.deprecate_version_2_0_0()

    assert (
        'Function "deprecate_version_2_0_0" is '
        "deprecated since version 2.0.0"
    ) in str(err)


def test_deprecate_version_2_1_0(DummyClass):
    with pytest.raises(RuntimeError) as err:
        obj = DummyClass()
        obj.deprecate_version_2_1_0()

    assert (
        'Function "deprecate_version_2_1_0" is '
        "deprecated since version 2.1.0"
    ) in str(err)


def test_deprecate_version_2_2_0(DummyClass):
    with pytest.warns(DeprecationWarning) as warning:
        obj = DummyClass()
        obj.deprecate_version_2_2_0()

    assert (
        'Function "deprecate_version_2_2_0" will '
        "be deprecated on version 2.2.0"
    ) in warning[0].message.args[0]

    with pytest.warns(DeprecationWarning) as warning:
        obj = DummyClass()
        obj.deprecate_property_2_2_0()

    assert (
        'Function "deprecate_property_2_2_0" will '
        "be deprecated on version 2.2.0"
    ) in warning[0].message.args[0]



import pytest

from auto_deprecator import deprecate


__version__ = "2.0.0"


@deprecate(
    expiry="2.1.0",
    version_module="tests.function.test_deprecate_version_module",
)
def simple_deprecate():
    pass


@deprecate(
    expiry="2.1.0", version_module="tests.function.conftest",
)
def failed_to_locate_version():
    pass


@deprecate(
    expiry="2.1.0", version_module="tests.function.not_existing_module",
)
def not_existing_module():
    pass


def test_no_error_simple_deprecate():
    with pytest.warns(DeprecationWarning) as warning:
        simple_deprecate()

    assert (
        'Function "simple_deprecate" will be deprecated on version 2.1.0'
    ) in warning[0].message.args[0]


def test_failed_to_locate_version():
    with pytest.raises(RuntimeError) as error:
        failed_to_locate_version()

    assert (
        "Cannot find version (__version__) from the version module "
        '"tests.function.conftest"'
    ) in str(error.value)


def test_not_existing_module():
    with pytest.raises(RuntimeError) as error:
        not_existing_module()

    assert (
        'Cannot locate version module "tests.function.not_existing_module"'
    ) in str(error.value)



import argparse
import ast
from os.path import isfile

from auto_deprecator.deprecate import check_deprecation


def check_import_deprecator_exists(tree, last_lineno):
    import_deprecator_lines = []

    for index, body in enumerate(tree.body):
        if (
            isinstance(body, (ast.ImportFrom, ast.Import))
            and body.module == "auto_deprecator"
        ):
            start_lineno = body.lineno

            if index != len(tree.body) - 1:
                end_lineno = tree.body[index + 1].lineno
            else:
                end_lineno = last_lineno

            import_deprecator_lines.append((start_lineno, end_lineno))

    return import_deprecator_lines


def check_body_deprecator_exists(body):
    if not hasattr(body, "decorator_list"):
        return None

    deprecate_list = [
        d for d in body.decorator_list if d.func.id == "deprecate"
    ]

    if len(deprecate_list) == 0:
        return None

    assert len(deprecate_list) == 1, (
        "More than one deprecate decorator is found "
        'in the function "{func}"'.format(func=body.func.name)
    )

    return deprecate_list[0]


def check_tree_deprecator_exists(tree):
    for body in tree.body:
        if isinstance(body, ast.ClassDef):
            if check_tree_deprecator_exists(body):
                return True

        if check_body_deprecator_exists(body):
            return True

    return False


def find_deprecated_lines(tree, current, begin_lineno, last_lineno):
    def get_function_lineno(body):
        if hasattr(body, "decorator_list") and len(body.decorator_list) > 0:
            return body.decorator_list[0].lineno
        else:
            return body.lineno

    deprecated_lines = []
    deprecated_body = []

    for index, body in enumerate(tree.body):
        # Python 3.8 lineno is on the function rather than the decorator
        start_lineno = get_function_lineno(body)

        if index != len(tree.body) - 1:
            end_lineno = get_function_lineno(tree.body[index + 1])
        else:
            end_lineno = last_lineno

        if isinstance(body, ast.ClassDef):
            deprecated_lines += find_deprecated_lines(
                body, current, start_lineno, last_lineno
            )

            if len(body.body) == 0:
                deprecated_body.append(body)

        deprecate_decorator = check_body_deprecator_exists(body)

        if deprecate_decorator is None:
            continue

        keywords = {k.arg: k.value.s for k in deprecate_decorator.keywords}

        expiry = (
            deprecate_decorator.args[0].value
            if deprecate_decorator.args
            else keywords["expiry"]
        )

        if current is None:
            current = (
                deprecate_decorator.args[1].value
                if len(deprecate_decorator.args) > 1
                else keywords["current"]
            )

        is_deprecated = check_deprecation(expiry=expiry, current=current)

        if not is_deprecated:
            continue

        # Readjust the start lineno from the decorator
        start_lineno = body.decorator_list[0].lineno

        deprecated_lines.append((start_lineno, end_lineno))
        deprecated_body.append(body)

    # Remove the deprecated body from the tree
    for body in deprecated_body:
        tree.body.remove(body)

    # If no element is found in the body, remove the whole tree
    if len(tree.body) == 0:
        deprecated_lines = [(begin_lineno, last_lineno)]

    return deprecated_lines


def deprecate_single_file(filename, current=None):
    # Read file stream
    filestream = open(filename, "r").readlines()
    tree = ast.parse("".join(filestream))

    # Check whether deprecate is included
    deprecator_import_lines = check_import_deprecator_exists(
        tree, len(filestream) + 1
    )

    if not deprecator_import_lines:
        return False

    # Store the deprecated funcion line numbers. The tuple
    # is combined by the start and end line index
    deprecated_lines = find_deprecated_lines(
        tree, current, 1, len(filestream) + 1
    )

    if not deprecated_lines:
        return False

    # Remove the import of the auto_deprecator if no more
    # deprecate decorator is found
    if not check_tree_deprecator_exists(tree):
        deprecated_lines += deprecator_import_lines

    # Remove the deprecated functions from backward
    deprecated_lines = sorted(
        deprecated_lines, key=lambda x: x[0], reverse=True
    )

    for start_lineno, end_lineno in deprecated_lines:
        filestream = (
            filestream[: start_lineno - 1] + filestream[end_lineno - 1 :]
        )

    # Remove the redundant newline
    filestream = "".join(filestream).rstrip()

    # Write back the file
    open(filename, "w+").write(filestream)

    return True


def deprecate_directory(path, current):
    raise NotImplementedError("Deprecate path does not support directory now")


def main():
    parser = argparse.ArgumentParser(
        description="Automatical removal of deprecated source code."
    )
    parser.add_argument("PATH", type=str, help="The source code path.")
    parser.add_argument(
        "--version", dest="current", type=str, help="Current package version."
    )
    args = parser.parse_args()

    # Get the argument values
    path = args.path
    current = args.current

    if isfile(path):
        deprecate_single_file(path, current)
    else:
        deprecate_directory(path, current)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = []

setup_requirements = [
    "pytest-runner",
]

test_requirements = [
    "pytest>=3",
]

setup(
    author="Gavin Chan",
    author_email="gavincyi@gmail.com",
    python_requires=">=3.5",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Python decorator and command to automate deprecating components",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="auto_deprecator",
    name="auto_deprecator",
    packages=find_packages(include=["auto_deprecator", "auto_deprecator.*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    extras_require={"test": test_requirements,},
    url="https://github.com/gavincyi/auto_deprecator",
    version="2020.2.0",
    zip_safe=False,
    entry_points="""
        [console_scripts]
        auto-deprecate = auto_deprecator.cli.auto_deprecate:main
    """,
)



"""Main module."""
from os import environ
from importlib import import_module
from warnings import warn


def deprecate(expiry=None, current=None, relocate=None, version_module=None):
    """Deprecate

    This is a function decorator which

      1. If the current version is before the expiry version, by deafult,
         a deprecation warning is raised.

      2. If the current version is same as or after the expiry version, by
         default, an exception is raised.

    :param expiry: `str` The expiry version, e.g. 2.1.0.
    :param current: `str` The current version. e.g. 2.0.0.
    :param relocate: `str` The relocated method or function name, which will
        be hinted if warning or exception is raised.
    :param version_module: `str` The module name which includes the current
        version (__version__).
    """

    def _deprecate(func):
        def wrapper(*args, **kwargs):
            # Check whether the function is deprecated
            is_deprecated = check_deprecation(
                expiry=expiry, current=current, version_module=version_module
            )

            # Throw exception if deprecation
            if is_deprecated:
                handle_deprecation(func=func, expiry=expiry, relocate=relocate)

            # Run the function
            result = func(*args, **kwargs)

            # Alert the user that the function will be
            # deprecated
            alert_future_deprecation(
                func=func, expiry=expiry, relocate=relocate
            )

            return result

        return wrapper

    return _deprecate


def get_curr_version(current, version_module):
    if "DEPRECATE_VERSION" in environ:
        return environ["DEPRECATE_VERSION"]

    assert (current is not None) or (version_module is not None), (
        "Only the current version (%s) or the version module (%s) "
        "should be specified"
    ) % (current, version_module)

    if current:
        return current

    try:
        module = import_module(version_module, "__version__")
    except ModuleNotFoundError:
        raise RuntimeError(
            'Cannot locate version module "%s"' % version_module
        )

    try:
        return getattr(module, "__version__")
    except AttributeError:
        raise RuntimeError(
            "Cannot find version (__version__) from the version module "
            '"%s"' % version_module
        )


def check_deprecation(expiry=None, current=None, version_module=None):
    if expiry is not None:
        current = get_curr_version(
            current=current, version_module=version_module
        )

        if current >= expiry:
            return True

    return False


def handle_deprecation(func, expiry=None, relocate=None):
    if expiry is None:
        return

    if relocate:
        hints = ' Please use function / method "{relocate}"'.format(
            relocate=relocate
        )
    else:
        hints = ""

    raise RuntimeError(
        (
            'Function "{func}" is deprecated since version {version}.'
            "{hints}"
        ).format(func=func.__name__, version=expiry, hints=hints)
    )


def alert_future_deprecation(func, expiry=None, relocate=None):
    if expiry is None:
        version_msg = "soon"
    else:
        version_msg = "on version {version}".format(version=expiry)

    if relocate:
        hints = ' Please use function / method "{relocate}"'.format(
            relocate=relocate
        )
    else:
        hints = ""

    warn(
        'Function "{func}" will be deprecated {version_msg}.{hints}'.format(
            func=func.__name__, version_msg=version_msg, hints=hints
        ),
        DeprecationWarning,
    )



import pytest

from auto_deprecator import deprecate


@deprecate()
def simple_deprecate():
    pass


def test_no_error_simple_deprecate():
    with pytest.warns(DeprecationWarning) as warning:
        simple_deprecate()

    assert (
        'Function "simple_deprecate" will ' "be deprecated soon"
    ) in warning[0].message.args[0]



import importlib.util
from tempfile import NamedTemporaryFile

import pytest


IMPORT_STATEMENT = r"""from auto_deprecator import deprecate


"""

CLASS_DELARATION = r"""class DummyClass:"""

INIT_METHOD = r"""

    def __init__(self):
        pass"""

DEPRECATE_2_0_0 = r"""

    @deprecate(expiry="2.0.0", current="2.1.0")
    def deprecate_version_2_0_0(self):
        pass"""

DEPRECATE_2_1_0 = r"""

    @deprecate(expiry="2.1.0", current="2.1.0")
    def deprecate_version_2_1_0(self):
        pass"""

DEPRECATE_2_2_0 = r"""

    @deprecate(expiry="2.2.0", current="2.1.0")
    def deprecate_version_2_2_0(self):
        pass"""

DEPRECATE_2_2_0_PROPERTY = r"""

    @deprecate(expiry="2.2.0", current="2.1.0")
    def deprecate_property_2_2_0(self):
        return 1"""

INNER_CLASS = r"""

    class DummyClass2:
        @deprecate(expiry="2.3.0", current="2.1.0")
        def deprecate_version_2_3_0(self):
            pass"""


@pytest.fixture(scope="session")
def dummy_class():
    return (
        IMPORT_STATEMENT
        + CLASS_DELARATION
        + INIT_METHOD
        + DEPRECATE_2_0_0
        + DEPRECATE_2_1_0
        + DEPRECATE_2_2_0
        + DEPRECATE_2_2_0_PROPERTY
        + INNER_CLASS
    )


@pytest.fixture
def dummy_class_file(dummy_class):
    with NamedTemporaryFile(mode="a+", suffix=".py") as tmpfile:
        tmpfile.write(dummy_class)
        tmpfile.flush()
        yield tmpfile.name


@pytest.fixture(name="DummyClass")
def dummy_class_fixture(dummy_class_file):
    spec = importlib.util.spec_from_file_location(
        "dummy_class", dummy_class_file
    )
    dummy_class = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(dummy_class)
    return dummy_class.DummyClass



from os import environ

import pytest


def test_deprecate_version_2_0_0_with_1_9_0(function_module):
    environ["DEPRECATE_VERSION"] = "1.9.0"

    with pytest.warns(DeprecationWarning) as warning:
        function_module.deprecate_version_2_0_0()

    del environ["DEPRECATE_VERSION"]

    assert (
        'Function "deprecate_version_2_0_0" will '
        "be deprecated on version 2.0.0"
    ) in warning[0].message.args[0]


def test_deprecate_version_2_0_0_with_2_1_0(function_module):
    environ["DEPRECATE_VERSION"] = "2.1.0"

    with pytest.raises(RuntimeError) as err:
        function_module.deprecate_version_2_0_0()

    del environ["DEPRECATE_VERSION"]

    assert (
        'Function "deprecate_version_2_0_0" is '
        "deprecated since version 2.0.0"
    ) in str(err)
